@@ -110,7 +110,9 @@ def trends(
     save
         save the plot.
     save_genes
-        save list of genes following the order displayed o nthe heatmap.
+        save list of genes following the order displayed on the heatmap.
+    **kwargs
+        arguments passed to :func:`scFates.pl.trajectory` if `plot_emb=True`
 
     Returns
     -------
@@ -410,11 +412,11 @@ def trends(
 
     axheatmap = add_frames(axheatmap, fitted_sorted.shape[0])
 
-    if highlight_features is "A":
+    if highlight_features == "A":
         highlight_features = (
             adata.var.A[features].sort_values(ascending=False)[:n_features].index
         )
-    elif highlight_features is "fdr":
+    elif highlight_features == "fdr":
         highlight_features = (
             adata.var.fdr[features].sort_values(ascending=True)[:n_features].index
         )
