@@ -1,4 +1,5 @@
-from .graph_tools import tree, curve, cleanup, root, roots, getpath
+from .graph_tools import tree, curve, cleanup, getpath
+from .root import root, roots
 from .pseudotime import pseudotime, refine_pseudotime, rename_milestones
 from .test_association import test_association
 from .fit import fit
@@ -10,6 +11,8 @@ from .correlation_tools import (
     module_inclusion,
 )
 from .correlation_tools import critical_transition, criticality_drivers
-from .bifurcation_tools import test_fork, branch_specific, activation
+from .bifurcation_tools import test_fork, branch_specific, activation, activation_lm
+
+# from .activation_tools import activation, activation_lm
 from .conversion import cellrank_to_tree
 from .linearity_deviation import linearity_deviation
